# -*- coding: utf-8 -*-
# @Time    : 2019/10/24 17:50
# @E-Mail  : aberstone.hk@gmail.com
# @File    : __version__.py
# @Software: PyCharm

# __title__ = 'bloomfilter'
__title__ = 'iron_man'
__description__ = 'C & Python Bloomfilter Based Memeroy || Redis .'
__version__ = '1.0.2'
__build__ = 0x022200
__author__ = 'Aberstone'
__author_email__ = 'aberstone.hk@gmail.com'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2019 Aberstone'
__cake__ = u'\u2728 \U0001f370 \u2728'
__url__ = "http://gitlab.nuist.pro:3000/aberstone/IronMan.git"